import javax.swing.ImageIcon; 
import javax.swing.JOptionPane; 
import database.*;

public class Quizspiel{
  //Anbindung an die Datenbank
  private DatabaseConnector connector;

  //ID-Nummer des aktuellen Spielers
  private String aktuelleSpielerID;

  //Aufgaben, die aus der Datenbank ausgelesen wurden und in Zufallsreihenfolge gespielt werden.
  private List<Aufgabe> offeneAufgaben;
  private int anzahlOffeneAufgaben;

  //Statistikdaten
  private int korrekteBearbeitungenSpiel, bearbeitungenSpiel;
  private int korrekteBearbeitungenGesamt, bearbeitungenGesamt;

  public Quizspiel(){
    //Aktuellen Nutzer initialisieren. 
    aktuelleSpielerID = null;
    korrekteBearbeitungenSpiel = 0;
    bearbeitungenSpiel = 0;
    korrekteBearbeitungenGesamt = 0;
    bearbeitungenGesamt = 0;

    //Verbindung zur Datenbank aufbauen.
    anzahlOffeneAufgaben = 0;
    connector = new DatabaseConnector("localhost","3306","Quizdatenbank","root","");
    if (connector.getErrorMessage() != null){
      JOptionPane.showMessageDialog(null,"Verbindung zur Datenbank nicht möglich. \n"+connector.getErrorMessage(),"Fehlermeldung",JOptionPane.ERROR_MESSAGE,null); 
      System.exit(0);
    }    

    //Feld fuer Aufgaben erstellen.
    offeneAufgaben = new List<Aufgabe>();
  }

  public void anmelden(String pBenutzer, String pPasswort){
    //ID fuer den Benutzer ermitteln.
    connector.executeStatement("SELECT SpielerID FROM Spieler WHERE Benutzername='" + pBenutzer + "' AND Passwort = '" + pPasswort + "'");
    QueryResult result = connector.getCurrentQueryResult();  

    //Ueberpruefen, ob Anmeldung erfolgreich war.
    if (result.getRowCount() == 1) {
      aktuelleSpielerID = result.getData()[0][0];

      //SQL-Anweisung: Statistische Daten aus der DB laden.
      String anweisung = 
        "SELECT SUM(hatBearbeitet.anzahlBearbeitungen), SUM(hatBearbeitet.anzahlKorrekteBearbeitungen) " + 
        "FROM hatBearbeitet "+
        "WHERE hatBearbeitet.SpielerID = " + aktuelleSpielerID;
      connector.executeStatement(anweisung);
      result = connector.getCurrentQueryResult(); 

      if (result.getData()[0][0] != null){
        bearbeitungenGesamt = Integer.parseInt(result.getData()[0][0]);
        korrekteBearbeitungenGesamt = Integer.parseInt(result.getData()[0][1]);
      }

    }
  }

  public boolean istAngemeldet(){    
    return aktuelleSpielerID != null;
  }

  public Aufgabe gibAufgabe(){
    //Ueberpruefen, ob noch offene Aufgaben in der Pufferliste sind.
    if (anzahlOffeneAufgaben == 0 && this.istAngemeldet()){
      //SQL-Anweisung: Aufgaben nach Anzahl der Bearbeitungen durch den aktuellen Spieler aufsteigend sortiert abfragen.
      String anweisung = 
        "SELECT Aufgabe.AufgabeID, Aufgabe.Frage, Aufgabe.KorrekteAntwort," + 
        "  Aufgabe.FalscheAntwortA, Aufgabe.FalscheAntwortB, Aufgabe.FalscheAntwortC " +
        "FROM Aufgabe LEFT JOIN " +  
        "(" +
        "  SELECT hatBearbeitet.AufgabeID, hatBearbeitet.anzahlBearbeitungen " +
        "  FROM hatBearbeitet " +
        "  WHERE hatBearbeitet.SpielerID = " + aktuelleSpielerID +
        ") AS tmp ON Aufgabe.AufgabeID = tmp.AufgabeID " +
        "ORDER BY tmp.anzahlBearbeitungen";
      connector.executeStatement(anweisung);
      QueryResult result = connector.getCurrentQueryResult(); 

      //Die zwanzig am wenigsten durch den aktuellen Spieler bearbeiteten Aufgaben auslesen.
      for (int i = 0; i < 20; i++){
        offeneAufgaben.append(
          new Aufgabe(
            result.getData()[i][0],result.getData()[i][1],result.getData()[i][2],result.getData()[i][3],result.getData()[i][4],result.getData()[i][5]
          )
        );
      }   
      anzahlOffeneAufgaben = 20;
    }

    //Zufallsaufgabe ermitteln.
    offeneAufgaben.toFirst();
    for (int i = 0; i < (int) (Math.random()*anzahlOffeneAufgaben); i++){
      offeneAufgaben.next();      
    }
    return offeneAufgaben.getContent();
  }

  public void abgebenAufgabe(Aufgabe pAufgabe){
    //Abgegebene Aufgabe aus der Liste der offenen Aufgaben entfernen.
    offeneAufgaben.toFirst();
    while (pAufgabe != offeneAufgaben.getContent()) {
      offeneAufgaben.next();
    }
    offeneAufgaben.remove();

    //Anzahl der noch offenen Fragen reduzieren.
    anzahlOffeneAufgaben = anzahlOffeneAufgaben - 1;

    //SQL-Anweisung: Anzahl der Eintraege zum aktuellen Spieler und der aktuellen Aufgabe aus der Verknuepfungstabelle ermitteln.
    String anweisung = 
      "SELECT Count(hatBearbeitet.SpielerID) " + 
      "FROM hatBearbeitet " + 
      "WHERE hatBearbeitet.SpielerID = " + aktuelleSpielerID + " AND hatBearbeitet.AufgabeID = " + pAufgabe.gibAufgabeID();
    connector.executeStatement(anweisung); 
    QueryResult result = connector.getCurrentQueryResult();  

    if (Integer.parseInt(result.getData()[0][0]) == 0){ //Neuen Eintrag in Verknüpfungstabelle eintragen.
      if (pAufgabe.korrektBeantwortet()) {
        anweisung = 
        "INSERT INTO hatBearbeitet(SpielerID, AufgabeID, anzahlBearbeitungen, anzahlKorrekteBearbeitungen) " + 
        "VALUES (" + aktuelleSpielerID +", "+ pAufgabe.gibAufgabeID() +", 1, 1)";        
        korrekteBearbeitungenSpiel = korrekteBearbeitungenSpiel + 1;     
        korrekteBearbeitungenGesamt = korrekteBearbeitungenGesamt + 1;
      } else {
        anweisung = 
        "INSERT INTO hatBearbeitet(SpielerID, AufgabeID, anzahlBearbeitungen, anzahlKorrekteBearbeitungen) " + 
        "VALUES (" + aktuelleSpielerID +", "+ pAufgabe.gibAufgabeID() +", 1, 0)";
      }
      connector.executeStatement(anweisung);
    } else { //Bestehenden Eintrag in Verknüpfungstabelle aktualisieren.
      //SQL-Anweisung: Anzahl der bisherigen Bearbeitungen generell und der korrekten Bearbeitungen abfragen.
      anweisung = 
      "SELECT hatBearbeitet.anzahlBearbeitungen, hatBearbeitet.anzahlKorrekteBearbeitungen " +
      "FROM hatBearbeitet " + 
      "WHERE hatBearbeitet.SpielerID = " + aktuelleSpielerID + " AND hatBearbeitet.AufgabeID ="+ pAufgabe.gibAufgabeID();
      connector.executeStatement(anweisung);      
      result = connector.getCurrentQueryResult();  

      //Werte entsprechen der aktuellen Bearbeitung aktualisieren.
      int anzahlBearbeitungen = Integer.parseInt(result.getData()[0][0]) + 1;
      int anzahlKorrekteBearbeitungen;
      if (pAufgabe.korrektBeantwortet()){
        anzahlKorrekteBearbeitungen = Integer.parseInt(result.getData()[0][1]) + 1;
        korrekteBearbeitungenSpiel = korrekteBearbeitungenSpiel + 1;
        korrekteBearbeitungenGesamt = korrekteBearbeitungenGesamt + 1;
      } else {
        anzahlKorrekteBearbeitungen = Integer.parseInt(result.getData()[0][1]);
      }

      //Neue Werte in die Datenbank uebertragen.
      anweisung = 
      "UPDATE hatBearbeitet SET anzahlBearbeitungen = " +  anzahlBearbeitungen + ", anzahlKorrekteBearbeitungen = " + anzahlKorrekteBearbeitungen + " " +
      "WHERE hatBearbeitet.SpielerID =" + aktuelleSpielerID + " AND hatBearbeitet.AufgabeID = " + pAufgabe.gibAufgabeID();
      connector.executeStatement(anweisung);
    }    

    bearbeitungenSpiel = bearbeitungenSpiel + 1;
    bearbeitungenGesamt = bearbeitungenGesamt + 1;

  } 

  public int gibKorrekteBearbeitungenGesamt(){
    return korrekteBearbeitungenGesamt;
  }

  public int gibBearbeitungenGesamt(){
    return bearbeitungenGesamt;
  }

  public int gibKorrekteBearbeitungenSpiel(){
    return korrekteBearbeitungenSpiel;
  }

  public int gibBearbeitungenSpiel(){
    return bearbeitungenSpiel;
  }

}

